# https://gist.github.com/armish/564a65ab874a770e2c26

# This function sorts the matrix for better visualization of mutual exclusivity across genes
memoSort <- function(M) {
	geneOrder <- sort(rowSums(M), decreasing=TRUE, index.return=TRUE)$ix;
	scoreCol <- function(x) {
		score <- 0;
		for(i in 1:length(x)) {
			if(x[i]) {
				score <- score + 2^(length(x)-i);
			}
		}
		return(score);
	}
	scores <- apply(M[geneOrder, ], 2, scoreCol);
	sampleOrder <- sort(scores, decreasing=TRUE, index.return=TRUE)$ix;
	return(M[geneOrder, sampleOrder]);
}
 
# This is the plotting function
oncoPrint <- function(M, colorVector, sort=TRUE) {
        mm = M
        mm[which(mm!="none")] = TRUE;
        mm[which(mm=="none")] = FALSE;
        mm=type.convert(mm);
	if(sort) {
		alts <- memoSort(mm);		
	} else {
		alts <- mm;
	}
 
	ngenes <- nrow(alts);
	nsamples <- ncol(alts);
	coverage <- sum(colSums(alts) > 0);
 
	### OncoPrint
	numOfOncos <- ngenes*nsamples;
	oncoCords <- matrix( rep(0, numOfOncos * 5), nrow=numOfOncos );
	colnames(oncoCords) <- c("xleft", "ybottom", "xright", "ytop", "altered");
 
	xpadding <- .01;
	ypadding <- .01;
	cnt <- 1;
	for(i in 1:ngenes) {
		for(j in 1:nsamples) {
			xleft <- j-1 + xpadding;
			ybottom <- ((ngenes-i+1) -1) + ypadding;
			xright <- j - xpadding;
			ytop <- (ngenes-i+1) -ypadding;
                        altered <- alts[i, j];
                        gene = rownames(alts)[i]
                        sample = colnames(alts)[j]
			oncoCords[cnt, ] <- c(xleft, ybottom, xright, ytop, M[gene,sample]);
			cnt <- cnt+1;
		}
	}
        
	colors <- rep("white", cnt);
    colors[ which(oncoCords[, "altered"] == "pos") ] <- "darkorange";
    colors[ which(oncoCords[, "altered"] == "neg") ] <- "lightgreen";
       
        
        #main=sprintf("Gene set altered in %.2f%%: %d of %d cases", coverage/nsamples*100, coverage, nsamples)
    par(oma=c(12,0,0,0))
	plot(c(0, nsamples), c(0, ngenes), type="n", main="" , xlab="", ylab="", yaxt="n",xaxt="n");
	rect(oncoCords[, "xleft"], oncoCords[, "ybottom"],oncoCords[, "xright"], oncoCords[, "ytop"], col=colors, border="white");        
    axis(2, at=(ngenes:1)-.5, labels=rownames(alts), las=2,cex.axis=0.55);
        
    col_vect = as.character(colorVector[rev(colnames(alts)),c(2)])
    axis(1, labels = FALSE, at=(nsamples:1)-.5, las=2,cex.axis=1.6);
    
    mtext(side = 1, text = rev(colnames(alts)), at = (nsamples:1)-.5, col = col_vect, line = 1,las=2,cex=0.9)
        legend("topright",  c("Positive correlation","Negative correlation"), cex=1.8,  fill=c("red","green"),bty="n");
        #legend("bottomright",  c("MSI", "MSI&LOH","HER","LOH","HER&LOH","Negative"), cex=1.2, col=c("darkmagenta","bisque","forestgreen","goldenrod","greenyellow","lightgray"),lty=1, lwd=5,bty="n");
}